#include "WAT128.h"



#ifdef _USE_SCI0_

void InitSCI0(UINT16 iBaud)
{
	UINT16 uiBaud = (MAIN_CLK/16/iBaud) -1;
	UCSR0A=0x00;
	UCSR0B=0x98;
	UCSR0C=0x06;
	UBRR0H=uiBaud>>8;
	UBRR0L=uiBaud>>0;

	structRX0Info.rx_wr_index1=0;
	structRX0Info.rx_rd_index1=0;

	structTX0Info.rd_index1=0;
	structTX0Info.wr_index1=0;
	structTX0Info.bComplete = TRUE;



}

ISR(USART0_RX_vect)				/* UART0 interrupt function */
{
	unsigned char status,uart_data;

	status=UCSR0A;
	uart_data = UDR0;			/* Read receive data */
	if ((status & (FRAMING_ERROR | PARITY_ERROR | DATA_OVERRUN))==0)
	{          
		if (structRX0Info.rx_wr_index1 >= RX_BUFFER_SIZE-1) structRX0Info.rx_wr_index1=0;
		structRX0Info.rx_buffer[structRX0Info.rx_wr_index1]=uart_data;
		structRX0Info.rx_wr_index1++; 
	}
}


INT16S GetByte0(void){
	INT16S iReturnValue = -1;
	cli();                              // global interrupt disable
	if(	structRX0Info.rx_rd_index1 >= structRX0Info.rx_wr_index1){
		structRX0Info.rx_rd_index1 = 0;
		structRX0Info.rx_wr_index1 = 0;
		iReturnValue = -1;
	}else{
		if(structRX0Info.rx_rd_index1 >= RX_BUFFER_SIZE-1){
			structRX0Info.rx_rd_index1 = 0;
		}
		iReturnValue = structRX0Info.rx_buffer[structRX0Info.rx_rd_index1];
		structRX0Info.rx_rd_index1++;
	}
	sei();                              // global interrupt enable
	return iReturnValue;
}


//!    
BOOL PutChar0(char c)
{
	BOOL	bReturn = FALSE;
	UINT16 iTemp=0;
	cli();                              // global interrupt disable
	structTX0Info.bComplete = FALSE;
	iTemp=0;
	while (!( UCSR0A & DATA_REGISTER_EMPTY)){
		if(iTemp++ >1000){
			bReturn =  FALSE;
			break;
		}
	}
	UDR0=c;
	sei();                              // global interrupt enable
	return bReturn;
}

void PutBytes0(BYTE *data, int iLength)
{
	int i=0;
	for(i=0;i<iLength;i++)
	{
		PutChar0(*(data+i));
	}


}

void PutData0(BYTE *data, int iLength)
{
	int i=0;
	BYTE byteCRC = 0;
	PutChar0(0x02);
	for(i=0;i<iLength;i++)
	{
		byteCRC +=*(data+i);
		PutChar0(*(data+i));
	}
	PutChar0(byteCRC);
	
	PutChar0(0x03);

}


//!  ڿ 
BOOL PutString0(char  *string ){
	BYTE i;
	i=0;
	while((string[i])!='\0' ){
		PutChar0(string[i]);
		i++;
	}
	return TRUE;
}


#endif


#ifdef _USE_SCI1_

void InitSCI1(UINT16 iBaud)
{
	UINT16 uiBaud = (MAIN_CLK/16/iBaud) -1;

	UCSR1A=0x00;
	UCSR1B=0x98;
	UCSR1C=0x06;
	UBRR1H=uiBaud>>8;
	UBRR1L=uiBaud>>0;

	structRX1Info.rx_wr_index1=0;
	structRX1Info.rx_rd_index1=0;

	structTX1Info.rd_index1=0;
	structTX1Info.wr_index1=0;
	structTX1Info.bComplete = TRUE;

}
#endif


#ifdef _USE_KEY4x4_

void InitKey4x4()
{
	KEY4x4_PORT_DIR = 0x0F;

}

UINT16 GetKey4x4()
{
	UINT16 uiReturnValue  =0;
	
	int i=0;
	// 
	for(i=3;i>=0;i--)
	{
		uiReturnValue <<= 4;
		KEY4x4_PORT = (1<<i);
		DelayUS(1);

		uiReturnValue |= (KEY4x4_PORT_IN>>4);
	}

	return uiReturnValue;
}

void SendKey4x4(UINT16 uidata)
{		
	BYTE byteData[10];
	int iDataLength=0;

	byteData[iDataLength++] = 0x11;
	byteData[iDataLength++] = uidata>>8;
	byteData[iDataLength++] = uidata;
	byteData[iDataLength++] = 0;
	byteData[iDataLength++] = 0;

	PutData0(&byteData,iDataLength);

}

#endif

#ifdef _USE_CLCD_

BYTE CLCD_BusyCheck()
{
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);
	DelayUS(200);


	return (0);

}


void CLCD_PutString(BYTE x, BYTE y,  char *str)
{
	int len = 0;

	CLCD_XY(x,y);

	while(*str ){
		if(len>16) break;
		CLCD_PutChar(*str++); 
		len++;
	}

}

void CLCD_PutChar(BYTE data)
{
	while(CLCD_BusyCheck());
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
	SetBit(CLCD_CONTROL_PORT,CLCD_RS);
	DelayUS(2);
	CLCD_DATA_PORT= data;
	DelayUS(50);
	SetBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(50);
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
}

void CLCD_Clear()
{
	CLCD_Command(1);
	DelayMS(1);
	CLCD_Command(2);
	DelayMS(1);        
}

void CLCD_XY(BYTE x, BYTE y)
{
	BYTE position = 0;
	if(x>16) x = 0;
	switch(y)
	{
	case 0 :
		position = 0x80;
		break;
	case 1 :
		position = 0xC0;
		break;
	}
	CLCD_Command(position + x);
}

void CLCD_Command(BYTE data)
{
	while(CLCD_BusyCheck()){}
	ClearBit(CLCD_CONTROL_PORT,CLCD_RS);
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(1);
	CLCD_DATA_PORT= data;
	DelayUS(50);
	SetBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(20);
	ClearBit(CLCD_CONTROL_PORT,CLCD_EA);
	DelayUS(50);
}

void CLCD_On()
{
	CLCD_Command(0x01);// dummy
	CLCD_Command(0x38);
	CLCD_Command(0x0C );
	CLCD_Command(0x01);
	CLCD_Command(0x06);
	CLCD_Command(0x01);
	DelayMS(2);
}

void CLCD_Init()
{
	SetBit(CLCD_CONTROL_PORT_DIR,CLCD_EA);
	SetBit(CLCD_CONTROL_PORT_DIR,CLCD_RS);
	SetBit(CLCD_CONTROL_PORT_DIR,CLCD_RW);

	ClearBit(CLCD_CONTROL_PORT,CLCD_RW);


	CLCD_DATA_PORT_DIR = 0xFF;


	CLCD_On();
	CLCD_Clear();
}


#endif

#ifdef _USE_ADC_
void InitADC()
{



		ACSR=0x00;
		SFIOR=0x00;

		ADCSRA=0x85 ;
		ADMUX=ADC_VREF_TYPE;
		DelayUS(250);
		DelayUS(250);
		ADCSRA = 0xD7;
	
}



#endif





#ifdef _USE_FND4_




void InitFND4()
{
	g_FND[0]=1;
	g_FND[1]=2;
	g_FND[2]=3;
	g_FND[3]=4;

	// FND DATA   (FND 4401ASR2 )
	FND4_DISPLAY_DATA[0] = 		0b00010100;//,		//0
	FND4_DISPLAY_DATA[1] = 			0b11010111;//,		//1
	FND4_DISPLAY_DATA[2] = 			0b01001100;//,		//2
	FND4_DISPLAY_DATA[3] = 			0b01000101;//,		//3
	FND4_DISPLAY_DATA[4] = 			0b10000111;//,		//4
	FND4_DISPLAY_DATA[5] = 			0b00100101;//,		//5
	FND4_DISPLAY_DATA[6] = 			0b00100100;//,		//6
	FND4_DISPLAY_DATA[7] = 			0b01010111;//,		//7
	FND4_DISPLAY_DATA[8] = 			0b00000100;//,		//8
	FND4_DISPLAY_DATA[9] = 			0b00000101;//		//9
	



	FND4_CONTROL_DIR |= 0xF0;
	FND4_DATA_DIR = 0xFF;

}






void DisplayFND4(BYTE b1,BYTE b2,BYTE b3,BYTE b4)
{

	SetBit(FND4_CONTROL_PORT,FND4_SEL1_BIT);
 	FND4_DATA = FND4_DISPLAY_DATA[b1];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL1_BIT);


	SetBit(FND4_CONTROL_PORT,FND4_SEL2_BIT);
	FND4_DATA = FND4_DISPLAY_DATA[b2];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL2_BIT);



	SetBit(FND4_CONTROL_PORT,FND4_SEL3_BIT);
	FND4_DATA = FND4_DISPLAY_DATA[b3];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL3_BIT);



	SetBit(FND4_CONTROL_PORT,FND4_SEL4_BIT);
	FND4_DATA = FND4_DISPLAY_DATA[b4];
	DelayMS(2);
	ClearBit(FND4_CONTROL_PORT,FND4_SEL4_BIT);

}





#endif



#ifdef _USE_ROTARY_

void InitRotary()
{
	ROTARY_DATA_DIR  &= 0xF0;

}

BYTE GetRotaryInt()
{
	return (~ROTARY_DATA_PORT & 0x0F);
}

#endif



void DelayUS(BYTE time_us)		/* time delay for us */
{ 
	register BYTE i;
	for(i = 0; i < time_us; i++)			// 4 cycle +
	{
		asm volatile(" PUSH  R0 ");		// 2 cycle +
		asm volatile(" POP   R0 ");		// 2 cycle +
		asm volatile(" PUSH  R0 ");		// 2 cycle +
		asm volatile(" POP   R0 ");		// 2 cycle +
		asm volatile(" PUSH  R0 ");		// 2 cycle +
		asm volatile(" POP   R0 ");		// 2 cycle = 16 cycle = 1 us for 16MHz
	}
}

void DelayMS(unsigned int time_ms)		/* time delay for ms */
{ 
	register unsigned int i;
	for(i = 0; i < time_ms; i++)
	{ 
		DelayUS(250);
		DelayUS(250);
		DelayUS(250);
		DelayUS(250);
	}
}
